// alarm.c
// Code for alarms
// Copyright (c) 4/2020 Bill Dally
//
// Copyright 2020 Bill Dally
// 
// Licensed under the Ventilator Open Source License (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
// ./LICENSE
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
#include "main.h"
#include "serial.h"
#include "solenoid.h"
#include "ui.h"

// Trigger an alarm - turn on beeper - display alarm on LCD line 2 (the LCD UI will do this)
void alarm_trig(uint16_t alarm_code) {
  alarm |= alarm_code  ; // on until reset - or codes into alarm in case there are multiple
  //PORTC |= 1<<ALARM_PC ; // turn on beeper - no longer used
  sol4_on() ; // turn on 110dB and relay
}

void flow_alarm(void) {
  alarm_trig(ALARM_FLOW) ;
  stop_vent() ;
}

void clear_alarm(void) {
  alarm = 0 ;
  //PORTC &= ~(1<<ALARM_PC) ; // turn alarm off - no longer used
  sol4_off() ;
}

uint8_t backup_hist_ptr(uint8_t ptr) {
  uint8_t val ;
  if(ptr == 0) val = (MAX_BPM-1) ;
  else val = ptr - 1 ;
  //put_word(ptr) ; space() ; put_word(val) ; crlf() ;
  return(val) ;
}

// put last breath into history array
void update_history(void) {
  b_hist_ptr += 1 ;
  if(b_hist_ptr >= MAX_BPM) b_hist_ptr = 0 ;
  if(b_hist_n_val < MAX_BPM) b_hist_n_val += 1 ; // keep track of valid entries
  b_hist[b_hist_ptr][BHIST_BVOL] = bvol ;
  b_hist[b_hist_ptr][BHIST_BPIP] = bpip ;
  b_hist[b_hist_ptr][BHIST_GTIME] = gtime ;
}

// returns 1 if gtime is less than a minute ahead of h_gtime
// accounts for wrap
int8_t less_than_minute(uint16_t gtime, uint16_t h_gtime) {
  int32_t delta ;
  //int16_t sdelta ;
  if(gtime >= h_gtime) { // no wrap
    delta = gtime - h_gtime ;
  } else { // wrap
    delta = ((int32_t) gtime + (1L<<16)) - (int32_t) h_gtime ;
  }
  // sdelta = delta ;
  //crlf() ; put_word(gtime) ; space() ; put_word(h_gtime) ; space() ;
  //put_word(sdelta) ; crlf() ;
  if(delta < GTICKS_PER_MIN)return(1) ;
  return(0) ;
}

void compute_minute_vol(void) {
  int8_t ptr, ltm, i ;
  uint16_t h_gtime ;
  minute_vol = 0 ;
  i = 0 ;
  ptr = b_hist_ptr ;
  //for(i=0 ; (i<bpm) ; i++) {
  do {
    minute_vol += b_hist[ptr][BHIST_BVOL] ;
    ptr = backup_hist_ptr(ptr) ;
    h_gtime = b_hist[ptr][BHIST_GTIME] ;
    ltm = less_than_minute(gtime, h_gtime) ;
    i += 1 ;
    // put_byte(ltm) ; crlf() ;
  } while(ltm && (i <= b_hist_n_val)) ;
}

void check_alarms(void) {
  update_history() ;

  // disconnect on low PIP for last 3 cycles
  if(bpip < pip_disconnect) disconnect_count += 1 ;
  else disconnect_count = 0 ;
  if(disconnect_count >= DISCONNECT_COUNT_THRESHOLD) alarm_trig(ALARM_DISCONNECT) ;

  // low/high minute volume
  // can't start until we've been running a minute
  if((b_hist_n_val >= bpm) || (gtime > GTICKS_PER_MIN)) {
    compute_minute_vol() ;
    if(minute_vol < low_minute_vol) alarm_trig(ALARM_LOW_MINUTE_VOL) ;
    if(minute_vol > high_minute_vol) alarm_trig(ALARM_HIGH_MINUTE_VOL) ;
  }

  // low input pressure
  if(in_p < ALARM_MIN_IN_P) alarm_trig(ALARM_IN_P) ;

  // low input voltage
  if(in_v < ALARM_MIN_IN_V) alarm_trig(ALARM_IN_V) ;
}

